@@ -77,7 +77,7 @@ try:
                        parameters_validation as pv,
                        iterables_manipulation as im,
                        multiprocessing_operations as mo)
-except:
+except ModuleNotFoundError:
     from CHEWBBACA.utils import (constants as ct,
                                  blast_wrapper as bw,
                                  file_operations as fo,
@@ -90,40 +90,38 @@ except:
 
 def bsr_categorizer(blast_results, representatives,
                     representatives_scores, min_bsr, max_bsr):
-    """ Determines the BLAST hits that have a BSR below a minimum threshold
-        and the BLAST hits that have a BSR above a maximum threshold.
+    """Identify BLAST hits below and above the BSR min and max thresholds.
 
-        Parameters
-        ----------
-        blast_results : list of list
-            A list with sublists, each sublist contains information
-            about a BLAST hit.
-        representatives : list
-            List with sequence identifiers of representative
-            sequences.
-        representatives_scores : dict
-            Dictionary with self BLAST raw score for every
-            representative.
-        min_bsr : float
-            Minimum BSR value accepted to consider a sequence as
-            a possible new representative.
-        max_bsr : float
-            Maximum BSR value accepted to consider a sequence as
-            a possible new representative.
+    Parameters
+    ----------
+    blast_results : list of list
+        A list with sublists, each sublist contains information
+        about a BLAST hit.
+    representatives : list
+        List with sequence identifiers of representative
+        sequences.
+    representatives_scores : dict
+        Dictionary with self BLAST raw score for every
+        representative.
+    min_bsr : float
+        Minimum BSR value accepted to consider a sequence as
+        a possible new representative.
+    max_bsr : float
+        Maximum BSR value accepted to consider a sequence as
+        a possible new representative.
 
-        Returns
-        -------
-        List with the following elements:
-            high_bsr : list
-                list with all sequence identifiers of subject
-                sequences that had hits with a BSR higher than the
-                maximum defined threshold.
-            low_bsr : list
-                list with all sequence identifiers of subject
-                sequences that had hits with a BSR lower than the
-                minimum defined threshold.
+    Returns
+    -------
+    List with the following elements:
+        high_bsr : list
+            list with all sequence identifiers of subject
+            sequences that had hits with a BSR higher than the
+            maximum defined threshold.
+        low_bsr : list
+            list with all sequence identifiers of subject
+            sequences that had hits with a BSR lower than the
+            minimum defined threshold.
     """
-
     high_bsr = []
     hotspot_bsr = []
     low_bsr = []
@@ -133,24 +131,24 @@ def bsr_categorizer(blast_results, representatives,
     low_reps = {}
 
     filtered_results = [res for res in blast_results
-                        if res[0] != res[1] and res[1] not in representatives]
-    bsr_values = [float(res[2])/float(representatives_scores[res[0]])
+                        if res[0] != res[4] and res[4] not in representatives]
+    bsr_values = [float(res[-1])/float(representatives_scores[res[0]])
                   for res in filtered_results]
 
-    high_bsr = [res[1] for ind, res in enumerate(filtered_results)
+    high_bsr = [res[4] for ind, res in enumerate(filtered_results)
                 if bsr_values[ind] >= max_bsr]
-    low_bsr = [res[1] for ind, res in enumerate(filtered_results)
+    low_bsr = [res[4] for ind, res in enumerate(filtered_results)
                if bsr_values[ind] < min_bsr]
-    hotspot_bsr = [res[1] for ind, res in enumerate(filtered_results)
+    hotspot_bsr = [res[4] for ind, res in enumerate(filtered_results)
                    if bsr_values[ind] >= min_bsr and bsr_values[ind] < max_bsr]
 
     for ind, res in enumerate(filtered_results):
         if bsr_values[ind] >= min_bsr:
-            high_reps.setdefault(res[0], []).append(res[1])
+            high_reps.setdefault(res[0], []).append(res[4])
         if bsr_values[ind] < min_bsr:
-            low_reps.setdefault(res[0], []).append(res[1])
+            low_reps.setdefault(res[0], []).append(res[4])
         if bsr_values[ind] >= min_bsr and bsr_values[ind] < max_bsr:
-            hot_reps.setdefault(res[0], []).append(res[1])
+            hot_reps.setdefault(res[0], []).append(res[4])
 
     # determine representatives that only led to low BSR
     low_reps = list(set(low_reps) - set(high_reps))
@@ -160,30 +158,29 @@ def bsr_categorizer(blast_results, representatives,
 
 def select_candidate(candidates, proteins, seqids,
                      representatives, final_representatives):
-    """ Chooses a new representative sequence.
+    """Select a new representative sequence.
 
-        Parameters
-        ----------
-        candidates : list
-            List with the sequence identifiers of all candidates.
-        proteins : dict
-            A dictionary with protein identifiers as keys and
-            protein sequences as values.
-        seqids : list
-            A list with the sequence identifiers that still have
-            no representative (representatives identifiers are
-            included because they have to be BLASTed in order to
-            determine their self score).
-        representatives : list
-            The sequence identifiers of all representatives.
+    Parameters
+    ----------
+    candidates : list
+        List with the sequence identifiers of all candidates.
+    proteins : dict
+        A dictionary with protein identifiers as keys and
+        protein sequences as values.
+    seqids : list
+        A list with the sequence identifiers that still have
+        no representative (representatives identifiers are
+        included because they have to be BLASTed in order to
+        determine their self score).
+    representatives : list
+        The sequence identifiers of all representatives.
 
-        Returns
-        -------
-        representatives : list
-            The set of all representatives, including the new
-            representative that was chosen by the function.
+    Returns
+    -------
+    representatives : list
+        The set of all representatives, including the new
+        representative that was chosen by the function.
     """
-
     # with more than one sequence as candidate, select longest
     if len(candidates) > 1:
 
@@ -229,44 +226,45 @@ def select_candidate(candidates, proteins, seqids,
 
 def adapt_loci(genes, schema_path, schema_short_path, bsr, min_len,
                table_id, size_threshold, blastp_path, makeblastdb_path):
-    """ Adapts a set of genes/loci from an external schema so that
-        that schema  can be used with chewBBACA. Removes invalid alleles
-        and selects representative alleles to include in the "short" directory.
+    """Adapts a set of genes/loci from an external schema.
 
-        Parameters
-        ----------
-        genes_list : list
-            A list with the following elements:
+    Adapts an external schema for usage with chewBBACA. Removes invalid
+    alleles and selects representative alleles to include in the "short"
+    directory.
 
-            - List with paths to the files to be processed.
-            - Path to the schema directory.
-            - Path to the "short" directory.
-            - BLAST Score Ratio value.
-            - Minimum sequence length value.
-            - Genetic code.
-            - Sequence size variation threshold.
+    Parameters
+    ----------
+    genes_list : list
+        A list with the following elements:
 
-        Returns
-        -------
-        invalid_alleles : list
-            List with the identifiers of the alleles that were
-            determined to be invalid.
-        invalid_genes : list
-            List with the identifiers of the genes that had no
-            valid alleles.
-        summary_stats : list of list
-            List with one sublist per processed locus. Each
-            sublist has four elements:
+        - List with paths to the files to be processed.
+        - Path to the schema directory.
+        - Path to the "short" directory.
+        - BLAST Score Ratio value.
+        - Minimum sequence length value.
+        - Genetic code.
+        - Sequence size variation threshold.
 
-            - The identifier of the locus.
-            - The number of alleles in the external file.
-            - The number of alleles that were a valid CDS.
-            - The number of representatives determined determined
-              by the process.
+    Returns
+    -------
+    invalid_alleles : list
+        List with the identifiers of the alleles that were
+        determined to be invalid.
+    invalid_genes : list
+        List with the identifiers of the genes that had no
+        valid alleles.
+    summary_stats : list of list
+        List with one sublist per processed locus. Each
+        sublist has four elements:
 
-        The function writes the schema files .
+        - The identifier of the locus.
+        - The number of alleles in the external file.
+        - The number of alleles that were a valid CDS.
+        - The number of representatives determined determined
+          by the process.
+
+    The function writes the schema files .
     """
-
     # divide input list into variables
     summary_stats = []
     invalid_genes = []
@@ -278,14 +276,13 @@ def adapt_loci(genes, schema_path, schema_short_path, bsr, min_len,
 
         # get gene basename and identifier
         gene_basename = os.path.basename(gene)
-        gene_id = gene_basename.split('.f')[0]
+        gene_id = gene_basename.split('.fasta')[0]
 
         # create paths to gene files in new schema
-        gene_file = fo.join_paths(schema_path,
-                                  ['{0}{1}'.format(gene_id, '.fasta')])
+        gene_file = fo.join_paths(schema_path, [gene_basename])
 
         gene_short_file = fo.join_paths(schema_short_path,
-                                        ['{0}{1}'.format(gene_id, '_short.fasta')])
+                                        [gene_basename.replace('.fasta', '_short.fasta')])
 
         # create path to temp working directory for current gene
         gene_temp_dir = fo.join_paths(schema_path,
@@ -323,8 +320,9 @@ def adapt_loci(genes, schema_path, schema_short_path, bsr, min_len,
             # create FASTA file with distinct protein sequences
             protein_file = fo.join_paths(gene_temp_dir,
                                          ['{0}_protein.fasta'.format(gene_id)])
-            protein_lines = fao.fasta_lines(ids_to_blast, prot_seqs)
-            fo.write_list(protein_lines, protein_file)
+            protein_data = [[i, prot_seqs[i]] for i in ids_to_blast]
+            protein_lines = fao.fasta_lines(ct.FASTA_RECORD_TEMPLATE, protein_data)
+            fo.write_lines(protein_lines, protein_file)
 
             # create blastdb with all distinct proteins
             blastp_db = os.path.join(gene_temp_dir, gene_id)
@@ -340,11 +338,12 @@ def adapt_loci(genes, schema_path, schema_short_path, bsr, min_len,
                 # create FASTA file with representative sequences
                 rep_file = fo.join_paths(gene_temp_dir,
                                          ['{0}_rep_protein.fasta'.format(gene_id)])
-                rep_protein_lines = fao.fasta_lines(representatives, prot_seqs)
-                fo.write_list(rep_protein_lines, rep_file)
+                rep_protein_data = [[r, prot_seqs[r]] for r in representatives]
+                rep_protein_lines = fao.fasta_lines(ct.FASTA_RECORD_TEMPLATE, rep_protein_data)
+                fo.write_lines(rep_protein_lines, rep_file)
 
                 # create file with seqids to BLAST against
-                ids_str = im.concatenate_list([str(i) for i in ids_to_blast], '\n')
+                ids_str = im.join_list([str(i) for i in ids_to_blast], '\n')
                 ids_file = fo.join_paths(gene_temp_dir,
                                          ['{0}_ids.txt'.format(gene_id)])
                 fo.write_to_file(ids_str, ids_file, 'w', '')
@@ -365,8 +364,8 @@ def adapt_loci(genes, schema_path, schema_short_path, bsr, min_len,
                 blast_results = fo.read_tabular(blast_output)
 
                 # get self-score for representatives
-                rep_self_scores = {res[1]: res[2] for res in blast_results
-                                   if res[0] == res[1]}
+                rep_self_scores = {res[0]: res[-1] for res in blast_results
+                                   if res[0] == res[4]}
 
                 # divide results into high, low and hot BSR values
                 hitting_high, hitting_low, hotspots, high_reps, low_reps, hot_reps = \
@@ -422,16 +421,18 @@ def adapt_loci(genes, schema_path, schema_short_path, bsr, min_len,
             final_representatives = list(prot_seqs.keys())
 
         # write schema file with all alleles
-        gene_lines = fao.fasta_lines(list(gene_seqs.keys()), gene_seqs)
-        fo.write_list(gene_lines, gene_file)
+        gene_data = [[k, v] for k, v in gene_seqs.items()]
+        gene_lines = fao.fasta_lines(ct.FASTA_RECORD_TEMPLATE, gene_data)
+        fo.write_lines(gene_lines, gene_file)
 
         # get total number of valid sequences
         valid_sequences = len(gene_lines)
 
         # write schema file with representatives
         final_representatives = [seqids_map[rep] for rep in final_representatives]
-        gene_rep_lines = fao.fasta_lines(final_representatives, gene_seqs)
-        fo.write_list(gene_rep_lines, gene_short_file)
+        gene_rep_data = [[r, gene_seqs[r]] for r in final_representatives]
+        gene_rep_lines = fao.fasta_lines(ct.FASTA_RECORD_TEMPLATE, gene_rep_data)
+        fo.write_lines(gene_rep_lines, gene_short_file)
 
         # get number of representatives
         representatives_number = len(gene_rep_lines)
@@ -470,7 +471,7 @@ def main(input_files, output_directory, cpu_cores, blast_score_ratio,
 
     # list schema gene files
     genes_file = pv.check_input_type(input_files,
-                                     os.path.join(output_directory, 'schema_genes.txt'))
+                                     fo.join_paths(output_directory, [ct.LOCI_LIST]))
 
     # import list of schema files
     with open(genes_file, 'r') as gf:
@@ -485,17 +486,16 @@ def main(input_files, output_directory, cpu_cores, blast_score_ratio,
     # count number of sequences and mean length per gene
     genes_info = []
     genes_pools = multiprocessing.Pool(processes=cpu_cores)
-    gp = genes_pools.map_async(fao.gene_seqs_info, genes_list,
+    gp = genes_pools.map_async(fao.fasta_stats, genes_list,
                                callback=genes_info.extend)
     gp.wait()
 
     # split files according to number of sequences and sequence mean length
     # in each file to pass even groups of sequences to all cores
-    even_genes_groups = mo.split_genes_by_core(genes_info, cpu_cores*4,
-                                               'seqcount')
+    # divide into 100 input sets to get 1% progress resolution
+    even_genes_groups = mo.distribute_loci(genes_info, 100, 'seqcount')
     # with few inputs, some sublists might be empty
     even_genes_groups = [i for i in even_genes_groups if len(i) > 0]
-
     # add common arguments
     blastp_path = os.path.join(blast_path, ct.BLASTP_ALIAS)
     makeblastdb_path = os.path.join(blast_path, ct.MAKEBLASTDB_ALIAS)
@@ -542,7 +542,7 @@ def main(input_files, output_directory, cpu_cores, blast_score_ratio,
     stats_lines = ['\t'.join(line) for line in stats_lines]
     stats_genes_file = '{0}/{1}_{2}'.format(schema_parent_directory,
                                             output_schema_basename,
-                                            'summary_stats.txt')
+                                            'summary_stats.tsv')
 
     with open(stats_genes_file, 'w') as stats:
         summary_stats_text = '\n'.join(stats_lines)
